/*
 * Copyright (C), 2020-2024, KylinSoft. Co., Ltd.
 * @Author: zhangfan
 * @FileName: unified_apis.h
 * @Date: 2024-06-14 14:01:20
 * @Version: v1.0
 * @LastEditors: zhangfan4 zhangfan4@kylinos.cn
 * @LastEditTime: 2024-09-04 14:08:17
 * @Description: 统一API的头文件，定义了统一接口的函数和数据结构
 * @History: Reference changelog
 * @Function List: 
 */
#ifndef __UNIFIED_APIS_H__
#define __UNIFIED_APIS_H__

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdint.h>
#include <json-c/json.h>
#include "kyee_security.h"
#include "tpm_security.h"

#ifdef DESKTOP

#if defined(__aarch64__)
#define DEFAULT_KYTRUST_CONFIG          "/boot/efi/boot/grub/.libkytrusted-security.conf"
#else
#define DEFAULT_KYTRUST_CONFIG          "/boot/grub/.libkytrusted-security.conf"
#endif

#else
#define DEFAULT_KYTRUST_CONFIG          "/boot/efi/EFI/kylin/.libkytrusted-security.conf"
#endif
#define TRUSTED_MODE_KYEE               "1"
#define TRUSTED_MODE_TPM                "2"
#define TRUSTED_MODE_SOFT               "3" 
#define TRUSTED_MODE_TCM2               "5"

#define TRUSTED_EFI_GUID_SIZE			35 


union TrustChainData
{
    /* kyee 信任链数据 */
    struct KYEE_TrustChainData kyee_trustChainData;
    /* TPM/TCM信任链数据 */
    struct TPM_TrustChainData tpm_trustChainData;
};

union Reportopt
{
    TpmReportOpt  tpm_opt;
    KyeeReportOpt kyee_opt;
};

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @brief 获取可信根数据
 * 
 * @param data [out]可信根数据(json格式)
 * @param nvpasswd_len [in]NV保护口令的长度
 * @param nvpasswd [in]NV保护口令
 * @return 0， success, 
 *         -2，仅可信根为SOFT时是success
 *         other, failed
 */
int TRUSTED_GetTrustedRootData(char *data, int nvpasswd_len, char * nvpasswd);
/**
 * @brief 获取系统目前配置的可信启动策略配置
 * 
 * @return  0 off(关闭)，3 warning(告警)，4 customized(拦截)
 */
int TRUSTED_GetBootMeasureCfgStatus();
/**
 * @brief 设置可信启动策略
 * 
 * @param status [in]可信启动策略模式状态值， 0 off(关闭)，3 warning(告警)，4 customized(拦截，只针对TPM/TCM2/soft可信根)
 * @param devicepath [in]可信设备的路径（NULL为选用默认设备）
 * @param trustrootmode [out]返回当前可信根模式
 * @param nvpasswd_len [in]NV保护口令的长度（若无，可传0）
 * @param nvpasswd [in]NV保护口令（若无，可传NULL）
 * @param authorization_len [in]可信根的所有者(Owner)授权数据的长度
 * @param authorization_para [in]可信根的所有者(Owner)授权数据
 * @return 0， 成功，-1， 失败
 */
int TRUSTED_SetBootMeasureCfgStatus(int status, char *devicepath, int *trustrootmode, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
/**
 * @brief 获取可信根的efi GUID
 * 
 * @param guid [out]可信根的efi GUID
 * @return 0， 成功，-1， 失败
 */
int TRUSTED_GetTrustedRootEfiGUID(char *guid);
/**
 * @brief 设置可信根的efi GUID，传入NULL为默认值607f766c-7455-42be-930be4d76db2720f
 * 
 * @param guidStr [in]可信根的efi GUID
 * @return 0， 成功，-1， 失败
 */
int TRUSTED_SetTrustedRootEfiGUID(char *guidStr);
/**
 * @brief 设置重新采集度量值
 *        kyee可信根需要重启系统更新基准值，TPM/TCM2/soft可信根执行函数立即更新基准值
 * 
 * @param status [in]重新采集度量值，该参数只对kyee可信根有效 (0,不采集; 1,重新采集)
 * @param filepath [in]文件路径（该参数只针对TPM/TCM2/soft可信根有效）
 * @param nvpasswd_len [in]NV保护口令的长度（若无，可传0）
 * @param nvpasswd [in]NV保护口令（若无，可传NULL）
 * @param authorization_len [in]可信根的所有者(Owner)授权数据的长度
 * @param authorization_para [in]可信根的所有者(Owner)授权数据
 * @return 0， 成功； -1， 失败
 */
int TRUSTED_SetReCollectMeasureStandarValue(int status,char *filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
/**
 * @brief 获取“下次启动是否会更新基准值”的标记状态（该接口只支持kyee可信根，因为非kyee可信根的更新基准值是立即更新）
 * 
 * @return 0 不采集; 1 重新采集; -1 失败 
 */
int TRUSTED_GetReCollectMeasureStandarValue();
/**
 * @brief 获取启动模式（该接口只支持kyee可信根）
 * 
 * @return  启动模式(0 同步； 1 混合；2 异步； 其他 失败)
 */
int TRUSTED_GetBootMode();
/**
 * @brief 设置启动模式（该接口只支持kyee可信根）
 * 
 * @param status [in]启动模式 (0 同步； 1 混合；2 异步)
 * @return 0， 成功；其他失败
 */
int TRUSTED_SetBootMode(int status); 
/**
 * @brief flash启动日志存储（该接口只支持kyee可信根）
 * 
 * @return 0， 成功；其他失败 
 */
int TRUSTED_SaveFlashBootMeasureLog(); 

/**
 * @brief 获取可信启动日志报告
 * 
 * @param reportopt [in]需要获取的报告类型（根据可信根类型选用TpmReportOpt或KyeeReportOpt）
 * @param list [out]可信启动日志报告数据 
 * @param count [out]数据数量
 * @return 0， 成功；其他失败
 */
int TRUSTED_GetBootMeasureData(int reportopt, void **list, int *count);
/**
 * @brief 获取历史启动报告（该接口目前只支持kyee可信根）
 * 
 * @param list [out]历史启动报告数据
 * @param count [out]数据数量
 * @return 0， 成功；其他失败 
 */
int TRUSTED_GetHistoryBootMeasureLog(void **list, int *count);

/**
 * @brief 获取当前的可信根类型：
 * 	1）若已存在可信启动配置，则从可信启动配置中返回可信根类型；
 *      2）若不存在可信启动配置，则通过调用可信根接口判断当前环境的软硬件支持何
 *         种可信根类型，优先级为kyee->TPM（TODO：后续支持TCM2和soft）。
 * @param trustMode [out]可信根类型（"1" kyee; "2" TPM; "3" soft；"5" TCM2）
 * @return 0， success,
 *       其它, failed
 */
int TRUSTED_GetCurrentUseTrustMode(char* trustMode);


//可信启动pks融合新增接口
/**
 * @brief 对该文件生成度量基准值，并将其添加到可信基准库中（该功能暂不支持kyee可信根，如当前为kyee可信模式则报错）
 * 
 * @param filepath [in]文件路径  
 * @param nvpasswd_len [in]NV保护口令的长度（若无，可传0）
 * @param nvpasswd [in]NV保护口令（若无，可传NULL）
 * @param authorization_len [in]可信根的所有者(Owner)授权数据的长度
 * @param authorization_para [in]可信根的所有者(Owner)授权数据
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_AddFile(char * filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);

/**
 * @brief 删除可信基准库中该文件的度量信息，并将其从可信基准库中删除（该功能不支持kyee可信根，如当前为kyee可信模式则报错）
 * 
 * @param filepath [in]文件路径
 * @param nvpasswd_len [in]NV保护口令的长度（若无，可传0）
 * @param nvpasswd [in]NV保护口令（若无，可传NULL）
 * @param authorization_len [in]可信根的所有者(Owner)授权数据的长度
 * @param authorization_para [in]可信根的所有者(Owner)授权数据
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_DealFile(char * filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);

/**
 * @brief 获取可信链的度量结果
 * 
 * @param data [out]可信链数据 
 *
 * @return 0， success, 
 *         other, failed
 */
int TRUSTED_GetTrustChainData(union TrustChainData* data);

#ifdef __cplusplus
}
#endif

#endif //__UNIFIED_APIS_H__
